/**
 * Configures.java
 * Created on 2005/05/16
 * Copyright (c) by es , All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package izayoi;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

/**
 * ݒێNX.<br>
 * 񋟂ĂœsꍇConfigurespNXāA
 * VM"Izayoi.configuresClass"ɂ̃NXLq邩A
 * getConfigures(java.lang.Class, java.io.File)g<br>
 * Ɏ̎w肪ȂƂ͕W
 * (Izayoi.PreferencesConfigAdapter)
 * @author es
 */
public abstract class Configures {

    private static Configures _instance;
    private static final String CONFIG_IMPLEMENTS = "Izayoi.configuresClass";

    /**
     * CX^X.<BR>
     * @param impl gpNX(nullVMˑ, VMȂꍇ͕W)
     * @param preferences l`t@C (null̏ꍇ͏l߂Ȃ)
     * @return Configures lŏꂽConfigures
     * @see izayoi.Configures#getConfigures(File)
     */
    synchronized private static final Configures createInstance(Class impl, File config) {
        try {
            if (impl == null) {
                String cls = System.getProperty(CONFIG_IMPLEMENTS, PreferencesConfigAdapter.class.getName());
                impl = Class.forName(cls);
            }
            _instance = (Configures) impl.getDeclaredConstructor(null).newInstance(null);
            _instance.init(config);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return _instance;
    }

    /**
     * ݒ擾.<br>
     * getConfigures(null)Ɠ
     * @return Configures
     */
    public static final Configures getConfigures() {
        return getConfigures(null);
    }

    /**
     * l`t@CAgpw肵Đݒ擾.<br>
     * ͎ˑ<br>
     * (xڈȍ~̌ďoł͈͖).
     * @param implementation gpConfigures
     * @param config l`t@C (ɏl߂Ȃꍇnull)
     * @return Configures
     */
    public static final Configures getConfigures(Class implementation, File config) {
        if (_instance != null) {
            return _instance;
        }
        return createInstance(implementation, config);
    }

    /**
     * `t@CŒlݒ擾.<br>
     * gpVMɈ˂.<br>
     * ͎ˑAW̏ꍇjava.util.prefs.PreferencesXML`<br>
     * (xڈȍ~̌ďoł͈͖).
     * @param config l`t@C (ɏl߂Ȃꍇnull)
     * @return Configures
     */
    public static final Configures getConfigures(File config) {
        return getConfigures(null, config);
    }

    /**
     * ݒl̎擾
     * @param name 
     * @return ݒl(݂Ȃꍇnull)
     */
    public final String getParameter(String name) {
        return getParameter(name, null);
    }

    /**
     * ݒlj
     * @throws IOException
     */
    abstract public void destroy() throws IOException;

    /**
     * ݒl̎擾
     * @param name 
     * @param defaultValue ݂Ȃꍇ̏l
     * @return ݒl(݂ȂꍇdefaultValue̒l)
     */
    abstract public String getParameter(String name, String defaultValue);

    /**
     * l`t@Cŏ.<br>
     * ͎ˑ
     * @param preferences l`t@C
     * @throws IOException
     */
    abstract protected void init(File config) throws IOException;

    /**
     * ݒl̒`
     * @param name 
     * @param value ݒl
     */
    abstract public void setParameter(String name, String value);

    /**
     * ݂̐ݒlt@Cɕۑ.<br>
     * ͎ˑ
     * @param config ۑ
     * @throws IOException
     */
    abstract public void store(File config) throws IOException;
}

/**
 * Configures̕W.<br>
 * java.util.prefs.PreferencesXML`g
 * @author es
 */
final class PreferencesConfigAdapter extends Configures {
    private final Preferences pref;

    PreferencesConfigAdapter() {
        pref = Preferences.systemRoot().node(Izayoi.NAME);
    }

    /* ( Javadoc)
     * @see izayoi.Configures#destroy()
     */
    public void destroy() throws IOException {
        try {
            Preferences.systemRoot().remove(Izayoi.NAME);
            Preferences.systemRoot().flush();
        } catch (BackingStoreException e) {
            IOException ex = new IOException(e.getMessage());
            ex.initCause(e);
            throw ex;
        }
    }

    /* ( Javadoc)
     * @see izayoi.Configures#getParameter(java.lang.String)
     */
    public String getParameter(String name, String defVal) {
        return pref.get(name, defVal);
    }

    /* ( Javadoc)
     * @see izayoi.Configures#init(java.io.File)
     */
    synchronized protected void init(File config) throws IOException {
        if (config == null) {
            return;
        }

        try {
            Preferences.importPreferences(new FileInputStream(config));
        } catch (Exception e) {
            if (e instanceof IOException) {
                throw (IOException) e;
            } else {
                IOException ex = new IOException(e.getMessage());
                ex.initCause(e);
                throw ex;
            }
        }
    }

    /* ( Javadoc)
     * @see izayoi.Configures#setParameter(java.lang.String, java.lang.String)
     */
    public void setParameter(String name, String value) {
        pref.put(name, value);
    }

    /* ( Javadoc)
     * @see izayoi.Configures#store(java.io.File)
     */
    synchronized public void store(File config) throws IOException {
        try {
            pref.exportNode(new FileOutputStream(config));
        } catch (BackingStoreException e) {
            IOException ex = new IOException(e.getMessage());
            ex.initCause(ex);
            throw ex;
        }
    }

}

/**
 * Configures̎.<br>
 * java.util.Properties`g
 * @author es
 */
final class PropertiesConfigAdapter extends Configures {
    private final Properties prop;

    PropertiesConfigAdapter() {
        prop = new Properties();
    }

    /* ( Javadoc)
     * @see izayoi.Configures#destroy()
     */
    public void destroy() throws IOException {
        //Ȃ
    }

    /* ( Javadoc)
     * @see izayoi.Configures#getParameter(java.lang.String, java.lang.String)
     */
    public String getParameter(String name, String defaultValue) {
        return prop.getProperty(name, defaultValue);
    }

    /* ( Javadoc)
     * @see izayoi.Configures#init(java.io.File)
     */
    synchronized protected void init(File config) throws IOException {
        if (config == null) {
            return;
        }
        prop.load(new FileInputStream(config));
    }

    /* ( Javadoc)
     * @see izayoi.Configures#setParameter(java.lang.String, java.lang.String)
     */
    public void setParameter(String name, String value) {
        synchronized (prop) {
            prop.setProperty(name, value);
        }
    }

    /* ( Javadoc)
     * @see izayoi.Configures#store(java.io.File)
     */
    synchronized public void store(File config) throws IOException {
        prop.store(new FileOutputStream(config), "Auto-generated by " + this.getClass().toString());
    }

}
