package izayoi.servlet;

import izayoi.Constants;
import izayoi.Util;
import izayoi.cache.CacheEntry;
import izayoi.cache.GenericRecord;
import izayoi.cache.Record;
import izayoi.cache.RecordSorter;
import izayoi.cache.WritableRecord;
import izayoi.exception.InvalidParameter;
import izayoi.view.TemplateContext;
import izayoi.view.ViewNote;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Replace of NoteCGI.java
 * @author es
 */
public class NoteServlet extends BasementServlet {

    /* ( Javadoc)
     * @see izayoi.servlet.BasementServlet#execute(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    protected void execute(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        String[] path = parseURL(request);
        if (path == null || path[0] == null) {
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
            return;
        }
        String filename = Util.urlDecode(path[0]);

        String command = request.getParameter(PARAM_COMMAND);
        if (command != null && command.equals(ACTION_POST) && request.getMethod().equalsIgnoreCase("post")) {
            postNote(request, response, filename);
        } else {
            getNote(request, response, filename, command);
        }
    }

    /**
     * m[g\
     * @param request
     * @param response
     * @param basename Ώۂ̃x[Xl[
     * @param command \[h
     * @throws IOException
     * @throws ServletException
     */
    private void getNote(HttpServletRequest request, HttpServletResponse response, String basename, String command)
        throws IOException, ServletException {
        CacheEntry entry = cache.getEntry(Constants.TYPE_NOTE, basename);

        ViewNote[] data = null;
        String rawtext = null;
        String baseid = null;
        long basetime = Long.MIN_VALUE;
        if (entry != null) {
            Record[] rec = null;
            if (command != null && command.equals(ACTION_HISTORY)) {
                rec = entry.getRecords(new RecordSorter.ByTimestamp());
            } else {
                rec = new Record[] { entry.getLastRecord()};
            }
			if(rec == null) {
				rec = new Record[0];
			}
            if(rec.length > 0) {

	            rawtext = rec[rec.length - 1].getField(Constants.FIELD_NAME_BODY);
	            if(rawtext == null) {
					rawtext = "";
	            }
	            rawtext = rawtext.replaceAll("<br>", "\r\n");
	            baseid = rec[rec.length - 1].getID();
	            basetime = rec[rec.length - 1].getTimeStamp();
			}

            data = new ViewNote[rec.length];
            for (int i = 0; i < rec.length; i++) {
                data[i] = new ViewNote(rec[i]);
            }
        }

        TemplateContext ctx = new TemplateContext("note.txt", request, response);
        ctx.setRecord(data);
        ctx.setName(basename);
        ctx.setSize(entry != null ? entry.getSize() : 0);
        ctx.put("rawtext", rawtext);
        ctx.put("baseid", baseid);
        ctx.put("basetime", basetime > 0 ? Long.toString(basetime) : null);
        ctx.put("form", new Boolean(command != null));
        ctx.finish();
    }

    /**
     * m[gҏW
     * @param request
     * @param response
     * @param basename Ώۂ̃x[Xl[
     */
    private void postNote(HttpServletRequest request, HttpServletResponse response, String basename)
        throws IOException, InvalidParameter {
        boolean isNotice = (request.getParameter(PARAM_DOPOST) != null);
        long basetime = Long.MIN_VALUE;
        try {
            basetime = Long.parseLong(request.getParameter(PARAM_BASETIME));
        } catch (Exception e) {
        }
        String baseid = request.getParameter(PARAM_BASEID);

        if (baseid == null || baseid.equals("")) {
            baseid = null;
            if(basetime != Long.MIN_VALUE) {
				throw new InvalidParameter(baseid, "l܂");
            }
        } else if(basetime == Long.MIN_VALUE) {
			throw new InvalidParameter(PARAM_BASETIME, "l܂");
		} else if (!baseid.matches("[a-f0-9]{32}")) {
			throw new InvalidParameter(PARAM_BASEID, baseid);
        }

        String body = Util.escape(request.getParameter(PARAM_BODY));
        if(body == null) {
        	body = "";
        }

		WritableRecord record = new GenericRecord();
        record.setTimeStamp(System.currentTimeMillis());
        record.setField(Constants.FIELD_NAME_BODY, body);
        if (basetime != Long.MIN_VALUE) {
            record.setField(Constants.FIELD_NAME_BASE_STAMP, String.valueOf(basetime));
        }
        if (baseid != null) {
            record.setField(Constants.FIELD_NAME_BASE_ID, baseid);
        }

        CacheEntry entry = cache.getEntry(Constants.TYPE_NOTE, basename, true);

        entry.append(record);
        if (!entry.isAutoCommit()) {
            entry.commit();
        }
        if (isNotice) {
			try {
				entry.noticeMe(record);
			} catch (IOException e1) { //Ȃ
			}
        }

        response.sendRedirect(request.getRequestURI());
    }

}
